@@ -2,7 +2,7 @@ import requests
 import os
 from mprop import mproperty
 
-__version__ = "0.86.2"
+__version__ = "0.91.4"
 
 __github_repo = "jupyter-naas/drivers"
 
@@ -344,3 +344,17 @@ def github():
     from naas_drivers.tools.github import Github
 
     return Github()
+
+
+@load_driver
+def bazimo():
+    from naas_drivers.tools.bazimo import Bazimo
+
+    return Bazimo()
+
+
+@load_driver
+def sharepoint():
+    from naas_drivers.tools.sharepoint import Sharepoint
+
+    return Sharepoint()
