@@ -5,6 +5,7 @@ import time
 import urllib
 from datetime import datetime
 import secrets
+import pydash as _pd
 
 LINKEDIN_API = "https://3hz1hdpnlf.execute-api.eu-west-1.amazonaws.com/prod"
 RELEASE_MESSAGE = (
@@ -15,6 +16,7 @@ RELEASE_MESSAGE = (
 DATE_FORMAT = "%Y-%m-%d"
 DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
 TIME_SLEEP = secrets.randbelow(3) + 2
+HEADERS = {"Content-Type": "application/json"}
 
 
 class LinkedIn(InDriver, OutDriver):
@@ -63,7 +65,7 @@ class LinkedIn(InDriver, OutDriver):
             occupation = occupation.strip().replace("\n", " ")
         return occupation
 
-    def connect(self, li_at: str, jessionid: str):
+    def connect(self, li_at: str = None, jessionid: str = None):
         # Init lk attribute
         self.li_at = li_at
         self.jessionid = jessionid
@@ -101,23 +103,58 @@ class Profile(LinkedIn):
         self.cookies = cookies
         self.headers = headers
 
-    def get_identity(self, url=None, urn=None):
+    def get_identity(self, url=None):
         res_json = {}
         result = {}
-        lk_id = self.get_profile_id(url)
-        req_url = f"https://www.linkedin.com/voyager/api/identity/profiles/{lk_id}"
+        lk_public_id = self.get_profile_id(url)
+        req_url = (
+            f"https://www.linkedin.com/voyager/api/identity/profiles/{lk_public_id}"
+        )
         res = requests.get(req_url, cookies=self.cookies, headers=self.headers)
         try:
             res.raise_for_status()
         except requests.HTTPError as e:
             print(e)
-        else:
-            res_json = res.json()
+        res_json = res.json()
         # Parse json
         data = res_json.get("data", {})
+        included = res_json.get("included", {})
+
+        # Init var
+        bg_pic_url = None
+        profile_pic_url = None
+
+        # Get data from included json
+        if len(included) > 0:
+            included = included[0]
+
+            # Get background picture
+            if included.get("backgroundImage"):
+                background_url_end = None
+                background_root = None
+                background_artifacts = _pd.get(included, "backgroundImage.artifacts")
+                if len(background_artifacts) > 0:
+                    background_url_end = background_artifacts[
+                        len(background_artifacts) - 1
+                    ].get("fileIdentifyingUrlPathSegment")
+                background_root = _pd.get(included, "backgroundImage.rootUrl")
+                if background_url_end and background_root:
+                    bg_pic_url = f"{background_root}{background_url_end}"
+
+            # Get profile picture
+            if included.get("picture"):
+                profile_url_end = None
+                profile_root = None
+                profile_artifacts = _pd.get(included, "picture.artifacts")
+                if len(profile_artifacts) > 0:
+                    profile_url_end = profile_artifacts[len(profile_artifacts) - 1].get(
+                        "fileIdentifyingUrlPathSegment"
+                    )
+                profile_root = _pd.get(included, "picture.rootUrl")
+                if profile_root and profile_url_end:
+                    profile_pic_url = f"{profile_root}{profile_url_end}"
+        lk_id = data.get("entityUrn", "").replace("urn:li:fs_profile:", "")
         result = {
-            "PROFILE_URN": data.get("entityUrn", "").replace("urn:li:fs_profile:", ""),
-            "PROFILE_ID": lk_id,
             "FIRSTNAME": data.get("firstName"),
             "LASTNAME": data.get("lastName"),
             "SUMMARY": data.get("summary"),
@@ -128,11 +165,16 @@ class Profile(LinkedIn):
             "COUNTRY": data.get("geoCountryName"),
             "LOCATION": data.get("locationName"),
             "BIRTHDATE": self.get_birthdate(data.get("birthDateOn")),
+            "PROFILE_ID": lk_id,
+            "PROFILE_URL": f"https://www.linkedin.com/in/{lk_id}",
+            "PUBLIC_ID": lk_public_id,
+            "BACKGROUND_PICTURE": bg_pic_url,
+            "PROFILE_PICTURE": profile_pic_url,
         }
         time.sleep(TIME_SLEEP)
         return pd.DataFrame([result])
 
-    def get_network(self, url=None, urn=None):
+    def get_network(self, url=None):
         res_json = {}
         result = {}
         lk_id = self.get_profile_id(url)
@@ -142,8 +184,7 @@ class Profile(LinkedIn):
             res.raise_for_status()
         except requests.HTTPError as e:
             print(e)
-        else:
-            res_json = res.json()
+        res_json = res.json()
         # Parse json
         data = res_json.get("data", {})
         result = {
@@ -159,7 +200,7 @@ class Profile(LinkedIn):
         time.sleep(TIME_SLEEP)
         return pd.DataFrame([result])
 
-    def get_contact(self, url=None, urn=None):
+    def get_contact(self, url=None):
         res_json = {}
         result = {}
         lk_id = self.get_profile_id(url)
@@ -169,10 +210,10 @@ class Profile(LinkedIn):
             res.raise_for_status()
         except requests.HTTPError as e:
             print(e)
-        else:
-            res_json = res.json()
+        res_json = res.json()
         # Parse json
         data = res_json.get("data", {})
+
         # Specific
         connected_at = data.get("connectedAt")
         if connected_at is not None:
@@ -222,8 +263,7 @@ class Profile(LinkedIn):
             if profile_urn is None:
                 return "Please enter a valid profile_url or profile_urn"
         req_url = f"{LINKEDIN_API}/profile/getResume?profile_urn={profile_urn}"
-        headers = {"Content-Type": "application/json"}
-        res = requests.post(req_url, json=self.cookies, headers=headers)
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
         try:
             res.raise_for_status()
         except requests.HTTPError as e:
@@ -329,8 +369,7 @@ class Profile(LinkedIn):
                 req_url = f"{LINKEDIN_API}/profile/getPostsFeed?profile_id={profile_id}&count={count}&pagination_token={pagination_token}"
             else:
                 req_url = f"{LINKEDIN_API}/profile/getPostsFeed?profile_id={profile_id}&count={count}"
-            headers = {"Content-Type": "application/json"}
-            res = requests.post(req_url, json=self.cookies, headers=headers)
+            res = requests.post(req_url, json=self.cookies, headers=HEADERS)
             try:
                 res.raise_for_status()
             except requests.HTTPError as e:
@@ -384,8 +423,7 @@ class Network(LinkedIn):
             if limit != -1 and limit < count:
                 count = limit
             req_url = f"{LINKEDIN_API}/network/getFollowers?start={start}&count={count}&limit={limit}"
-            headers = {"Content-Type": "application/json"}
-            res = requests.post(req_url, json=self.cookies, headers=headers)
+            res = requests.post(req_url, json=self.cookies, headers=HEADERS)
             try:
                 res.raise_for_status()
             except requests.HTTPError:
@@ -413,8 +451,7 @@ class Network(LinkedIn):
             if limit != -1 and limit < count:
                 count = limit
             req_url = f"{LINKEDIN_API}/network/getConnections?start={start}&count={count}&limit={limit}"
-            headers = {"Content-Type": "application/json"}
-            res = requests.post(req_url, json=self.cookies, headers=headers)
+            res = requests.post(req_url, json=self.cookies, headers=HEADERS)
             try:
                 res.raise_for_status()
             except requests.HTTPError:
@@ -441,13 +478,155 @@ class Invitation(LinkedIn):
         self.cookies = cookies
         self.headers = headers
 
+    def get(self, start=0, count=100):
+        """
+        Return an dataframe object with 15 columns:
+        - PROFILE_ID
+        - PUBLIC_ID
+        - FIRSTNAME
+        - LASTNAME
+        - FULLNAME
+        - OCCUPATION
+        - PROFILE_PICTURE
+        - MESSAGE
+        - SENT_AT
+        - UNSEEN
+        - INVITATION_TYPE
+        - INVITATION_DESC
+        - INVITATION_STATUS
+        - INVITATION_ID
+        - SHARED_SECRET
+
+        Parameters
+        ----------
+        response: str (default 'accept')
+            "accept" or "ignore"
+
+        invitation_id: str (default None)
+            Argument given when get invitations
+
+        invitation_shared_secret: int (default 1, max 100):
+            Number of requests sent to LinkedIn API.
+            (!) If count > 1, published date will not be returned.
+
+        is_generic: int (default 10, unlimited=-1):
+            Number of posts return by function. It will start with the most recent post.
+        """
+        req_url = f"{LINKEDIN_API}/invitation/get?start={start}&count={count}"
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
+        try:
+            res.raise_for_status()
+        except requests.HTTPError as e:
+            return e
+        else:
+            res_json = res.json()
+        return pd.DataFrame(res_json)
+
+    def response(
+        self,
+        action="accept",
+        invitation_id=None,
+        invitation_shared_secret=None,
+        is_generic=False,
+    ):
+        """
+        Print result : "Accept" or "Ignore"
+        Return dataframe profile if invitation type = "Profile"
+
+        Parameters
+        ----------
+        action: str (default 'accept')
+            "accept" or "ignore"
+
+        invitation_id: str (default None)
+            Argument given in invitations.get : "INVITATION_ID"
+
+        invitation_shared_secret: str (default None)
+            Argument given in invitations.get : "SHARED_SECRET"
+
+        is_generic: boolean (default False):
+            Must be True for generic invitation, if "INVITATION_TYPE" != "Profile"
+        """
+        params = {
+            "action": action,
+            "invitation_id": invitation_id,
+            "invitation_shared_secret": invitation_shared_secret,
+            "is_generic": is_generic,
+        }
+        req_url = f"{LINKEDIN_API}/invitation/response?{urllib.parse.urlencode(params, safe='(),')}"
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
+        try:
+            res.raise_for_status()
+        except requests.HTTPError as e:
+            return e
+        else:
+            res_json = res.json()
+        if action == "accept":
+            print("🤝 Invitation accepted !")
+        elif action == "ignore":
+            print("❌ Invitation ignored !")
+        return pd.DataFrame(res_json)
+
+    def accept(
+        self, invitation_id=None, invitation_shared_secret=None, is_generic=False
+    ):
+        """
+        Return dataframe profile if invitation type = "Profile"
+
+        Parameters
+        ----------
+        invitation_id: str (default None)
+            Argument given in invitations.get : "INVITATION_ID"
+
+        invitation_shared_secret: str (default None)
+            Argument given in invitations.get : "SHARED_SECRET"
+
+        is_generic: boolean (default False):
+            Must be True for generic invitation, if "INVITATION_TYPE" != "Profile"
+        """
+        action = "accept"
+        try:
+            df = self.response(
+                action, invitation_id, invitation_shared_secret, is_generic
+            )
+        except requests.HTTPError as e:
+            return e
+        return df
+
+    def ignore(
+        self, invitation_id=None, invitation_shared_secret=None, is_generic=False
+    ):
+        """
+        Return dataframe profile if invitation type = "Profile"
+
+        Parameters
+        ----------
+        invitation_id: str (default None)
+            Argument given in invitations.get : "INVITATION_ID"
+
+        invitation_shared_secret: str (default None)
+            Argument given in invitations.get : "SHARED_SECRET"
+
+        is_generic: boolean (default False):
+            Must be True for generic invitation, if "INVITATION_TYPE" != "Profile"
+        """
+        action = "ignore"
+        try:
+            df = self.response(
+                action, invitation_id, invitation_shared_secret, is_generic
+            )
+        except requests.HTTPError as e:
+            return e
+        return df
+
     def send(self, recipient_url=None, message="", recipient_urn=None):
         if recipient_url is not None:
             recipient_urn = self.get_profile_urn(recipient_url)
         if recipient_urn is None:
             return True
         if message:
-            message = ',"message":' '"' + message + '"'
+            message = ',"message":"' + message + '"'
+            message = ""
         data = (
             (
                 '{"trackingId":"yvzykVorToqcOuvtxjSFMg==","invitations":[],"excludeInvitations":[],'
@@ -488,11 +667,10 @@ class Message(LinkedIn):
             "limit": limit_max if limit > limit_max or limit == -1 else limit,
             "count": count,
         }
-        headers = {"Content-Type": "application/json"}
         df_result = None
         while True:
             req_url = f"{LINKEDIN_API}/message/getConversations?{urllib.parse.urlencode(params, safe='(),')}"
-            res = requests.post(req_url, json=self.cookies, headers=headers)
+            res = requests.post(req_url, json=self.cookies, headers=HEADERS)
             if limit != -1:
                 limit -= limit_max
                 if limit < 0:
@@ -531,8 +709,7 @@ class Message(LinkedIn):
             req_url += f"&conversation_url={conversation_url}"
         if conversation_urn:
             req_url += f"&conversation_urn={conversation_urn}"
-        headers = {"Content-Type": "application/json"}
-        res = requests.post(req_url, json=self.cookies, headers=headers)
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
         try:
             res.raise_for_status()
         except requests.HTTPError:
@@ -651,8 +828,7 @@ class Post(LinkedIn):
             if activity_id is None:
                 return "Please enter a valid post_url or activity_id"
         req_url = f"{LINKEDIN_API}/post/getStats?activity_id={activity_id}"
-        headers = {"Content-Type": "application/json"}
-        res = requests.post(req_url, json=self.cookies, headers=headers)
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
         try:
             res.raise_for_status()
         except requests.HTTPError as e:
@@ -689,10 +865,8 @@ class Post(LinkedIn):
             activity_id = LinkedIn.get_activity_id(post_url)
             if activity_id is None:
                 return "Please enter a valid post_url or activity_id"
-
         req_url = f"{LINKEDIN_API}/post/getPolls?activity_id={activity_id}"
-        headers = {"Content-Type": "application/json"}
-        res = requests.post(req_url, json=self.cookies, headers=headers)
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
         try:
             res.raise_for_status()
         except requests.HTTPError as e:
@@ -702,8 +876,7 @@ class Post(LinkedIn):
 
     def get_comments(self, post_url):
         req_url = f"{LINKEDIN_API}/post/getComments?post_link={post_url}"
-        headers = {"Content-Type": "application/json"}
-        res = requests.post(req_url, json=self.cookies, headers=headers)
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
         try:
             res.raise_for_status()
         except requests.HTTPError:
@@ -715,8 +888,7 @@ class Post(LinkedIn):
 
     def get_likes(self, post_url):
         req_url = f"{LINKEDIN_API}/post/getLikes?post_link={post_url}"
-        headers = {"Content-Type": "application/json"}
-        res = requests.post(req_url, json=self.cookies, headers=headers)
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
         try:
             res.raise_for_status()
         except requests.HTTPError:
@@ -735,8 +907,7 @@ class Event(LinkedIn):
 
     def get_guests(self, url):
         req_url = f"{LINKEDIN_API}/event/getGuests?event_link={url}"
-        headers = {"Content-Type": "application/json"}
-        res = requests.post(req_url, json=self.cookies, headers=headers)
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
         try:
             res.raise_for_status()
         except requests.HTTPError:
@@ -755,8 +926,7 @@ class Company(LinkedIn):
 
     def get_info(self, company_url):
         req_url = f"{LINKEDIN_API}/company/getInfo?company_url={company_url}"
-        headers = {"Content-Type": "application/json"}
-        res = requests.post(req_url, json=self.cookies, headers=headers)
+        res = requests.post(req_url, json=self.cookies, headers=HEADERS)
         try:
             res.raise_for_status()
         except requests.HTTPError as e:
